/*****************************************************************************/
/*                                                                           */
/*     Licensed Materials - Property of IBM                                  */
/*                                                                           */
/*     5622-078                                                              */
/*     (C) Copyright IBM Corp. 1988, 1995 All Rights Reserved.               */
/*                                                                           */
/*     US Government Users Restricted Rights - Use, duplication or           */
/*     disclosure restricted by GSA ADP Schedule Contract with               */
/*     IBM Corp.                                                             */
/*                                                                           */
/*****************************************************************************/
/*****************************************************************************/
/*                                                                           */
/*   Module Name: SF.H                                                       */
/*                                                                           */
/*   Data Information Area Subfield symbols and structures used by CM at     */
/*   both the CMI and GCCI interface (Includes CCITT, Network Specific, and  */
/*   Status subfields).                                                      */
/*                                                                           */
/*****************************************************************************/





#ifndef _SF_H
#define _SF_H

#ifndef _32_INCL
  #include "wcl1632.h"
#endif






/*****************************************************************************/
/*                      Common Subfield Information                          */
/*****************************************************************************/

/* Subfield Identifier */
#define SF_SFID_CCITT              0x80
#define SF_SFID_NETWORK_SPECIFIC   0x81
#define SF_SFID_CALL_CONTROL       0x82
#define SF_SFID_SUPPLEMENTAL_SRV   0x83
#define SF_SFID_PRM_SPECIFIC       0x04
#define SF_SFID_PROD_SPECIFIC      0x02

#define SF_PS_TYPE_PROT_SPECIFIC   0x01

#define SF_SFID_CNM_DUMP           0x40
#define SF_SFID_CNM_TRACE          0x41
#define SF_SFID_CNM_PROBLEM        0x42

#define SF_VALUE_NOT_PROVIDED      0xFF



/*****************************************************************************/
/*                                                                           */
/*                          CCITT Subfields                                  */
/*                                                                           */
/*****************************************************************************/
/*****************************************************************************/
/*                         Bearer Capability                                 */
/*****************************************************************************/

typedef STRUCT16 SF_CC_Bear_Cap
   {
      WORD              SFLength;       /* sizeof(SF_CC_BEAR_CAP)            */
      BYTE              SFIdentifier;   /* SF_SFID_CCITT                     */
      BYTE              SFType;         /* SF_CC_TYPE_BEAR_CAP               */
      BYTE              SFCallType;     /*                                   */
      BYTE              SFUserRate;     /*                                   */
      DWORD             SFUserRateVal;  /* User provided rate (hex)          */
      BYTE              SFSynch;        /* Synchronicity                     */
      BYTE              SFEncoding;     /* Encoding Scheme                   */
      WORD              SFModem;        /* modem class                       */
      BYTE              SFFramingStd;   /*                                   */
      BYTE              SFNetDataSize;  /*                                   */
   } SF_CC_BEAR_CAP;

typedef STRUCT16 SF_CC_Bear_Cap_Isdn
   {
      WORD              SFLength;       /* sizeof(SF_CC_BEAR_CAP)            */
      BYTE              SFIdentifier;   /* SF_SFID_CCITT                     */
      BYTE              SFType;         /* SF_CC_TYPE_BEAR_CAP               */
      BYTE              SFCallType;     /*                                   */
      BYTE              SFUserRate;     /*                                   */
   } SF_CC_BEAR_CAP_ISDN;

/* Subfield Type */
#define SF_CC_TYPE_BEAR_CAP             0x04

/* Call Type */
#define SF_CALL_TYPE_DATA_CIRCUIT       0x02
#define SF_CALL_TYPE_DATA_PACKET        0x03


/* User Rate */
/* The SFUserRate field in BC sf is reserved across CMI and the field will   */
/* set to 0x00 by Conn. Manager                                              */
#define SF_USER_RATE_UNSPECIFIED        0x00
#define SF_USER_RATE_64KBPS             0x00
#define SF_USER_RATE_56KBPS             0x01
#define SF_USER_RATE_IN_VALUE_FIELD     0xFE

/* Synchronicity */
#define SF_SYNC_UNSPECIFIED             0x00
#define SF_SYNC_ASYNCHRONOUS            0x01
#define SF_SYNC_SYNCHRONOUS             0x02
#define SF_SYNC_AUTOSYNC                0x04

/* Encoding Scheme */
#define SF_ENCODE_UNSPECIFIED           0x00
#define SF_ENCODE_NRZ                   0x01
#define SF_ENCODE_NRZI                  0x02

/* Modem Class    */
#define SF_MODEM_UNSPECIFIED            0x0000
#define SF_MODEM_V22                    0x0001
#define SF_MODEM_V22BIS                 0x0002
#define SF_MODEM_V32                    0x0003
#define SF_MODEM_V32BIS                 0x0004
#define SF_MODEM_PROPRIETARY            0x00FE

/* Framing Standard*/
#define SF_FRAMING_UNSPECIFIED          0x00
#define SF_FRAMING_ISO3309              0x01
#define SF_FRAMING_AWP0224_WITH_NEG     0x02
#define SF_FRAMING_AWP0224_WITHOUT_NEG  0x03

/* Network Data Size */
#define SF_NET_DATA_SIZE_UNSPECIFIED    0x00
#define SF_NET_DATA_SIZE_8BIT           0x01
#define SF_NET_DATA_SIZE_7BIT           0x02



/*****************************************************************************/
/*                         Channel Identification                            */
/*****************************************************************************/

typedef STRUCT16 SF_CC_Channel_Id
   {
      WORD             SFLength;        /* sizeof(SF_CC_CHANNEL_ID)          */
      BYTE             SFIdentifier;    /* SF_SFID_CCITT                     */
      BYTE             SFType;          /* SF_CC_TYPE_CHANNEL_ID             */
      BYTE             SFChannelPref;   /*                                   */
      BYTE             SFDChannelInd;   /*                                   */
      WORD             SFChannelSel;    /*                                   */
      DWORD            SFBMacAddr;      /* Address of B Mac associated with  */
      BYTE             SFChanMap[12];   /* up to 96 channels supported       */
      DWORD            SFConnID;        /* Connection ID                     */
   } SF_CC_CHANNEL_ID;

typedef struct SF_CC_Channel_Id_Isdn
   {
      WORD             SFLength;        /* sizeof(SF_CC_CHANNEL_ID)          */
      BYTE             SFIdentifier;    /* SF_SFID_CCITT                     */
      BYTE             SFType;          /* SF_CC_TYPE_CHANNEL_ID             */
      BYTE             SFChannelPref;   /*                                   */
      BYTE             SFDChannelInd;   /*                                   */
      BYTE             SFChannelSel;    /*                                   */
      BYTE             SFReserved;      /*                                   */
      DWORD            SFBMacAddr;      /* Address of B Mac associated with  */
                                        /*  selected channel (returned)      */
   } SF_CC_CHANNEL_ID_ISDN;

/* Subfield Type */
#define SF_CC_TYPE_CHANNEL_ID           0x18

/* Channel Preference */
#define SF_CHANNEL_PREFERRED            0x00
#define SF_CHANNEL_REQUIRED             0x01

/* D Channel Indicator */
#define SF_B_CHANNEL_IDENTIFIED         0x00
#define SF_D_CHANNEL_IDENTIFIED         0x01
#define SF_SEL_CHANNEL_IDENTIFIED       0x02

/* Channel Selection */
#define SF_CHANNEL_NO_CHANNEL           0x0000
#define SF_CHANNEL_B1                   0x0001
#define SF_CHANNEL_B2                   0x0002
#define SF_CHANNEL_ANY_CHANNEL          0x0003
#define SF_CHANNEL_IN_SLOT_MAP          0x0100

/* Channel MAP       */
/* with 8 channel map BYTES, up to 96 channels can be supported. Each bit*/
/* represents the selection of a single channel with byte 1, bit 1       */
/* identifying channel 1, up to byte 8, bit 8 representing channel 92.   */



/*****************************************************************************/
/*                         Calling Party Number                              */
/*****************************************************************************/

typedef STRUCT16 SF_CC_Calling_Number
   {
      WORD              SFLength;       /* sizeof(SF_CC_CALLING_NUMBER)      */
      BYTE              SFIdentifier;   /* SF_SFID_CCITT                     */
      BYTE              SFType;         /* SF_CC_TYPE_CALLING_NUMBER         */
      BYTE              SFNumberType;   /*                                   */
      BYTE              SFNumberPlan;   /*                                   */
      BYTE              SFPresentation; /*                                   */
      BYTE              SFScreening;    /*                                   */
      BYTE              SFNumberLength; /* Length of SFNumber array          */
      BYTE              SFNumber[1];    /* Number Digits (IA5 characters)    */
   } SF_CC_CALLING_NUMBER;

/* Subfield Type */
#define SF_CC_TYPE_CALLING_NUMBER       0x6C

/* Number Type */
#define SF_NUMBER_TYPE_UNKNOWN          0x00    /* changed from unspec       */
#define SF_NUMBER_TYPE_INTERNATIONAL    0x01
#define SF_NUMBER_TYPE_NATIONAL         0x02
#define SF_NUMBER_TYPE_NETWORK_SPECIFIC 0x03
#define SF_NUMBER_TYPE_SUBSCRIBER       0x04
#define SF_NUMBER_TYPE_ABBREVIATED      0x06

/* Numbering Plan */
#define SF_NUMBER_PLAN_UNKNOWN          0x00
#define SF_NUMBER_PLAN_ISDN             0x01
#define SF_NUMBER_PLAN_PRIVATE          0x09

/* Presentation Indicator */
#define SF_PRESENTATION_ALLOWED         0x00
#define SF_PRESENTATION_RESTRICTED      0x01
#define SF_PRESENTATION_NOT_AVAILABLE   0x02

/* Screening Indicator */
#define SF_SCREENING_NOT_SCREENED       0x00
#define SF_SCREENING_PASSED             0x01
#define SF_SCREENING_FAILED             0x02
#define SF_SCREENING_NETWORK_PROVIDED   0x03



/*****************************************************************************/
/*                         Called Party Number                               */
/*****************************************************************************/

typedef STRUCT16 SF_CC_Called_Number
   {
      WORD              SFLength;       /* sizeof(SF_CC_CALLED_NUMBER)       */
      BYTE              SFIdentifier;   /* SF_SFID_CCITT                     */
      BYTE              SFType;         /* SF_CC_TYPE_CALLED_NUMBER          */
      BYTE              SFNumberType;   /*                                   */
      BYTE              SFNumberPlan;   /*                                   */
      BYTE              SFNumberLength; /* Length of SFNumber array          */
      BYTE              SFNumber[1];    /* Number Digits (IA5 characters)    */
   } SF_CC_CALLED_NUMBER;

/* Subfield Type */
#define SF_CC_TYPE_CALLED_NUMBER        0x70

/* Number Type */
#define SF_NUMBER_TYPE_UNKNOWN          0x00    /* changed from unspec       */
#define SF_NUMBER_TYPE_INTERNATIONAL    0x01
#define SF_NUMBER_TYPE_NATIONAL         0x02
#define SF_NUMBER_TYPE_NETWORK_SPECIFIC 0x03
#define SF_NUMBER_TYPE_SUBSCRIBER       0x04
#define SF_NUMBER_TYPE_ABBREVIATED      0x06

/* Numbering Plan */
#define SF_NUMBER_PLAN_UNKNOWN          0x00
#define SF_NUMBER_PLAN_ISDN             0x01
#define SF_NUMBER_PLAN_PRIVATE          0x09



/*****************************************************************************/
/*                       Calling Party Subaddress                            */
/*****************************************************************************/

typedef STRUCT16 SF_CC_Calling_Subaddress
   {
      WORD              SFLength;       /* sizeof(SF_CC_CALLING_SUBADDRESS)  */
      BYTE              SFIdentifier;   /* SF_SFID_CCITT                     */
      BYTE              SFType;         /* SF_CC_TYPE_CALLING_SUBADDRESS     */
      BYTE              SFSubaddrType;  /*                                   */
      BYTE              SFAFIForNSAP;   /* 0x50 hex for IA5                  */
      BYTE              SFOddEvenInd;   /* 0x00 for even, 0x01 for odd       */
      BYTE              SFSubaddrLength;/* Length of SFSubaddr array         */
      BYTE              SFSubaddr[1];   /* Number Digits (IA5 characters)    */
   } SF_CC_CALLING_SUBADDRESS;

/* Subfield Type */
#define SF_CC_TYPE_CALLING_SUBADDRESS   0x6D

/* Subaddress Type */
#define SF_SUBADDRESS_TYPE_NSAP         0x00
#define SF_SUBADDRESS_USER_SPECIFIED    0x20

/* AFIForNSAP */
#define SF_AFI_FOR_IA5_CHAR             0x50

/* OddEvenInd */
#define SF_ODD_BCD                      0x01
#define SF_EVEN_BCD                     0x00



/*****************************************************************************/
/*                       Called Party Subaddress                             */
/*****************************************************************************/

typedef STRUCT16 SF_CC_Called_Subaddress
   {
      WORD              SFLength;       /* sizeof(SF_CC_CALLED_SUBADDRESS)   */
      BYTE              SFIdentifier;   /* SF_SFID_CCITT                     */
      BYTE              SFType;         /* SF_CC_TYPE_CALLED_SUBADDRESS      */
      BYTE              SFSubaddrType;  /*                                   */
      BYTE              SFAFIForNSAP;   /* 0x50 hex for IA5                  */
      BYTE              SFOddEvenInd;   /* 0x00 for even, 0x01 for odd       */
      BYTE              SFSubaddrLength;/* Length of SFSubaddr array         */
      BYTE              SFSubaddr[1];   /* Number Digits (IA5 characters)    */
   } SF_CC_CALLED_SUBADDRESS;

/* Subfield Type */
#define SF_CC_TYPE_CALLED_SUBADDRESS    0x71

/* Subaddress Type */
#define SF_SUBADDRESS_TYPE_NSAP         0x00
#define SF_SUBADDRESS_USER_SPECIFIED    0x20



/*****************************************************************************/
/*                       Low Layer Compatability                             */
/*****************************************************************************/

typedef STRUCT16 SF_CC_Low_Layer_Comp
   {
      WORD              SFLength;       /* sizeof(SF_CC_LOW_LAYER_COMP)      */
      BYTE              SFIdentifier;   /* SF_SFID_CCITT                     */
      BYTE              SFType;         /* SF_CC_TYPE_LOW_LAYER_COMP         */
      BYTE              SFLayer2Prot;   /*                                   */
      BYTE              SFLayer2User;   /* Available for user specification  */
      BYTE              SFLayer3Prot;   /*                                   */
      BYTE              SFLayer3User;   /* Available for user specification  */
   } SF_CC_LOW_LAYER_COMP;

/* Subfield Type */
#define SF_CC_TYPE_LOW_LAYER_COMP       0x7C

/* Layer 2 Protocol */
#define SF_LAYER2_PROT_UNSPECIFIED      0x40
#define SF_LAYER2_PROT_ISO_1475         0x41    /* high bits changed         */
#define SF_LAYER2_PROT_Q_921            0x42
#define SF_LAYER2_PROT_X_25_LINK_LVL    0x46
#define SF_LAYER2_PROT_X_25_MULTI_LINK  0x47
#define SF_LAYER2_PROT_EXTENDED_LAPB    0x48
#define SF_LAYER2_PROT_HDLC_ARM         0x49
#define SF_LAYER2_PROT_HDLC_NRM         0x4A
#define SF_LAYER2_PROT_HDLC_ABM         0x4B
#define SF_LAYER2_PROT_LAN_LLC          0x4C
#define SF_LAYER2_PROT_X_75_SLP         0x4D
#define SF_LAYER2_PROT_Q922             0x4E
#define SF_LAYER2_PROT_USER_SPECIFIC    0x50
#define SF_LAYER2_PROT_ISO_7776         0x51

/* Layer 2 User - 7820 communication on AT&T 5ESS */
#define SF_LAYER2_USER_7820_PORT_0      0x80
#define SF_LAYER2_USER_7820_PORT_1      0x81
#define SF_LAYER2_USER_7820_PORT_2      0x82

/* Layer 2 User - 7820 communication on 1TR6 */
#define SF_LAYER2_USER_7820_V35         0x81
#define SF_LAYER2_USER_7820_X21         0x82

/* Layer 3 Protocol */
#define SF_LAYER3_PROT_UNSPECIFIED      0x60
#define SF_LAYER3_PROT_Q_931            0x62    /* high bits changed         */
#define SF_LAYER3_PROT_X_25_PACKET      0x66
#define SF_LAYER3_PROT_ISO_8208         0x67
#define SF_LAYER3_PROT_ISO_8348         0x68
#define SF_LAYER3_PROT_ISO_8473         0x69
#define SF_LAYER3_PROT_CCITT_T_70       0x6A
#define SF_LAYER3_PROT_USER_SPECIFIC    0x70

/* Layer 3 User */
#define SF_LAYER3_USER_SNA              0x61    /* DCR 1851 */
#define SF_LAYER3_USER_NETBIOS          0x62    /* DCR 1851 */
#define SF_LAYER3_USER_TCP_IP           0x63    /* DCR 1851 */



/*****************************************************************************/
/*                      High Layer Compatability                             */
/*****************************************************************************/

typedef STRUCT16 SF_CC_High_Layer_Comp
   {
      WORD              SFLength;       /* sizeof(SF_CC_HIGH_LAYER_COMP)     */
      BYTE              SFIdentifier;   /* SF_SFID_CCITT                     */
      BYTE              SFType;         /* SF_CC_TYPE_HIGH_LAYER_COMP        */
      BYTE              SFCodingStd;    /*                                   */
      BYTE              SFHLC1;         /*                                   */
   } SF_CC_HIGH_LAYER_COMP;

/* Deleted SFHLCLength and SFHLC2 fields from the HLC sf -                   */
/* Subfield Type */
#define SF_CC_TYPE_HIGH_LAYER_COMP      0x7D

/* Coding Standard */
#define SF_CODING_STD_CCITT             0x00
#define SF_CODING_STD_INTERNATIONAL     0x01
#define SF_CODING_STD_NATIONAL          0x02
#define SF_CODING_STD_NETWORK_SPECIFIC  0x03

/* High Layer Characteristics - HLC1   */
#define SF_HLC1_TELEPHONY               0x01
#define SF_HLC1_FAX_GROUP2_3            0x04
#define SF_HLC1_T_503                   0x21
#define SF_HLC1_T_501                   0x24
#define SF_HLC1_T_502                   0x28
#define SF_HLC1_TELETEX                 0x31
#define SF_HLC1_T_504                   0x32
#define SF_HLC1_TELEX                   0x35
#define SF_HLC1_MESSAGE_HANDLING_SYS    0x38
#define SF_HLC1_OSI_APPLICATION         0x41



/*****************************************************************************/
/*                      User-User Information                                */
/*****************************************************************************/

typedef STRUCT16 SF_CC_User_User_Info
   {
      WORD              SFLength;       /* sizeof(SF_CC_USER_USER_INFO)      */
      BYTE              SFIdentifier;   /* SF_SFID_CCITT                     */
      BYTE              SFType;         /* SF_CC_TYPE_USER_USER_INFO         */
      BYTE              SFProtDisc;     /*                                   */
      BYTE              SFUserLength;   /* Length of UserInfo Array          */
      BYTE              SFUserInfo[1];  /* Available for user specification  */
   } SF_CC_USER_USER_INFO;

/* Subfield Type */
#define SF_CC_TYPE_USER_USER_INFO       0x7E

/* Protocol Discriminator */
#define SF_PROT_DISC_USER_SPECIFIED     0x00
#define SF_PROT_DISC_USER_OSI_HLP       0x01
#define SF_PROT_DISC_USER_X_244         0x02
#define SF_PROT_DISC_USER_IA5_CHARS     0x04
#define SF_PROT_DISC_USER_V_120         0x07
#define SF_PROT_DISC_USER_ISDN_CC       0x08



/*****************************************************************************/
/*                      User Facility                                        */
/*****************************************************************************/

typedef STRUCT16 SF_CC_User_Facility
   {
      WORD              SFLength;          /* sizeof(SF_CC_USER_FACILITY)     */
      BYTE              SFIdentifier;      /* SF_SFID_CCITT                   */
      BYTE              SFType;            /* SF_CC_TYPE_USER_FACILITY        */
      BYTE              SFReserved[128];   /* Reserved                        */
      BYTE              SFFacLength;       /* Length of User Facility array   */
      BYTE              SFFacInfo[1];      /* Facility Information            */
   } SF_CC_USER_FACILITY;

/* Subfield Type */
#define SF_CC_TYPE_USER_FACILITY        0x2C



/*****************************************************************************/
/*                           KeyPad                                          */
/*****************************************************************************/


/*****************************************************************************/
/*                           Progress Indicator                              */
/*****************************************************************************/

typedef STRUCT16 SF_CC_Progress_Ind
   {
      WORD              SFLength;       /* sizeof(SF_CC_PROGRESS_IND)        */
      BYTE              SFIdentifier;   /* SF_SFID_CCITT                     */
      BYTE              SFType;         /* SF_CC_TYPE_PROGRESS_IND           */
      BYTE              SFCodingStd;    /*                                   */
      BYTE              SFReserved1;    /* 0                                 */
      WORD              SFProgressDesc; /*                                   */
   } SF_CC_PROGRESS_IND;

/* Subfield Type */
#define SF_CC_TYPE_PROGRESS_IND         0x1E

/* Coding Standard */
#define SF_CODING_STD_CCITT             0x00
#define SF_CODING_STD_INTERNATIONAL     0x01
#define SF_CODING_STD_NATIONAL          0x02
#define SF_CODING_STD_NETWORK_SPECIFIC  0x03

/* Progess Description */
#define SF_PROGRESS_NOT_END_TO_END_ISDN 0x0001
#define SF_PROGRESS_DEST_NOT_ISDN       0x0002
#define SF_PROGRESS_ORIG_NOT_ISDN       0x0003
#define SF_PROGRESS_CALL_RETURN_TO_ISDN 0x0004
#define SF_PROGRESS_IN_BAND_INFO_AVAIL  0x0008



/*****************************************************************************/
/*                                Signal                                     */
/*****************************************************************************/

/*****************************************************************************/
/*                                Display                                    */
/*****************************************************************************/

typedef STRUCT16 SF_CC_Display
   {
      WORD              SFLength;       /* sizeof(SF_CC_DISPLAY)             */
      BYTE              SFIdentifier;   /* SF_SFID_CCITT                     */
      BYTE              SFType;         /* SF_CC_TYPE_DISPLAY                */
      BYTE              SFDisplayLength;/* Length of Display Array           */
      BYTE              SFDisplay[1];   /* IA5 characters                    */
   } SF_CC_DISPLAY;

/* Subfield Type */
#define SF_CC_TYPE_DISPLAY              0x28



/*****************************************************************************/
/*                                Cause                                      */
/*****************************************************************************/

typedef STRUCT16 SF_CC_Cause
   {
      WORD              SFLength;       /* variable                          */
      BYTE              SFIdentifier;   /* SF_SFID_CCITT                     */
      BYTE              SFType;         /* SF_CC_TYPE_CAUSE                  */
      BYTE              SFCodingStd;    /*                                   */
      BYTE              SFRecommend;    /*                                   */
      BYTE              SFLocation;     /*                                   */
      BYTE              SFReserved;     /*                                   */
      WORD              SFCause;        /* Network Specific Cause Value      */
      BYTE              SFAddlInfo[1];  /* Additional User Information       */
   } SF_CC_CAUSE;

/* Subfield Type */
#define SF_CC_TYPE_CAUSE                0x80

/* Coding Standard */
#define SF_CODING_STD_CCITT             0x00
#define SF_CODING_STD_INTERNATIONAL     0x01
#define SF_CODING_STD_NATIONAL          0x02
#define SF_CODING_STD_NETWORK_SPECIFIC  0x03

/* Recommendation */
#define SF_REC_ISDN_CALL_CONTROL        0x00
#define SF_REC_X_21                     0x03
#define SF_REC_X_25                     0x04

/* Location */
#define SF_LOC_USER                     0x00
#define SF_LOC_PRIVATE_NET_LOCAL_USER   0x01
#define SF_LOC_PUBLIC_NET_LOCAL_USER    0x02
#define SF_LOC_TRANSIT_NET              0x03
#define SF_LOC_PUBLIC_NET_REMOTE_USER   0x04
#define SF_LOC_REMOTE_PRIVATE_NET       0x05
#define SF_LOC_INTERNATIONAL_NET        0x07
#define SF_LOC_BEYOND_INTERWORKING_NET  0x0A

/* CCITT standard Cause Values */
/* Cause values in CCI_RC.H                                                  */



/*****************************************************************************/
/*                            Charging Information                           */
/*****************************************************************************/

typedef STRUCT16 SF_NS_Charging_Infor
   {
      WORD              SFLength;       /* sizeof(SF_NS_CHARGING_INFO)       */
      BYTE              SFIdentifier;   /* SF_SFID_NETWORK_SPECIFIC          */
      BYTE              SFType;         /* SF_NS_TYPE_CHARGING_INFO          */
      BYTE              SFChargeLength; /* Length of ChargeInfo array        */
      BYTE              SFChargeInfo[1];/* Charging Information              */
   } SF_NS_CHARGING_INFO;

/* Subfield Type */
#define SF_NS_TYPE_CHARGING_INFO        0x02



/*****************************************************************************/
/*                                 Date                                      */
/*****************************************************************************/

typedef STRUCT16 SF_NS_Date
   {
      WORD              SFLength;       /* sizeof(SF_NS_DATE)                */
      BYTE              SFIdentifier;   /* SF_SFID_NETWORK_SPECIFIC          */
      BYTE              SFType;         /* SF_NS_TYPE_DATE                   */
      BYTE              SFDate[14];     /* Date Information                  */
   } SF_NS_DATE;

/* Subfield Type */
#define SF_NS_TYPE_DATE                 0x03



/*****************************************************************************/
/*                            Call Status Subfield                           */
/*****************************************************************************/

typedef STRUCT16 SF_CS_Call_Status
   {
      WORD              SFLength;       /* sizeof(SF_CS_CALL_STATUS)         */
      BYTE              SFIdentifier;   /* SF_SFID_CALL_CONTROL              */
      BYTE              SFType;         /* SF_CS_TYPE_CALL_STATUS            */
      BYTE              SFCallStatus;   /*                                   */
      BYTE              SFReserved1;    /* 0                                 */
   } SF_CS_CALL_STATUS;

/* Subfield Type */
#define SF_CS_TYPE_CALL_STATUS          0x01

/* Call Status Value */
#define SF_STATUS_SETUP_ACK             0x00
#define SF_STATUS_CALL_PROC             0x01
#define SF_STATUS_CALL_ALERT            0x02
#define SF_STATUS_ACCEPT_CALL           0x03
#define SF_STATUS_STATUS                0x04
#define SF_STATUS_CALL_INFO             0x10



/*****************************************************************************/
/*                            Unformatted Message                            */
/*****************************************************************************/

typedef STRUCT16 SF_CS_Raw_Msg
   {
      WORD              SFLength;       /* sizeof(SF_CS_RAW_MSG)             */
      BYTE              SFIdentifier;   /* SF_SFID_CALL_CONTROL              */
      BYTE              SFType;         /* SF_CS_TYPE_RAW_MSG                */
      BYTE              SFRawMsg[1];    /*                                   */
   } SF_CS_RAW_MSG;

/* Subfield Type */
#define SF_CS_TYPE_RAW_MSG              0x02



/*****************************************************************************/
/*                            Permanent Connection Subfield                  */
/*****************************************************************************/

typedef STRUCT16 SF_CS_Permanent
   {
      WORD              SFLength;       /* sizeof(SF_CS_PERMANENT)           */
      BYTE              SFIdentifier;   /* SF_SFID_CALL_CONTROL              */
      BYTE              SFType;         /* SF_CS_TYPE_PERMANENT              */
      BYTE              SFRole;         /*                                   */
      BYTE              SFReserved;     /* SF_CS_TYPE_PERMANENT              */
      BYTE              SFPermName[32]; /*                                   */
   } SF_CS_PERMANENT;

/* Subfield Type */
#define SF_CS_TYPE_PERMANENT            0x03

/* Role           */
#define SF_ROLE_ORIGINATE               0x01
#define SF_ROLE_ANSWER                  0x02



/*****************************************************************************/
/*                            CSIF Subfield                                  */
/*****************************************************************************/

typedef STRUCT16 SF_CC_Csif
   {
      WORD              SFLength;       /* sizeof(SF_CC_CSIF)                */
      BYTE              SFIdentifier;   /* SF_SFID_CALL_CONTROL              */
      BYTE              SFType;         /* SF_CC_TYPE_CSIF                   */
      DWORD             SFSockBindID;   /* Socket bind id for this call      */
      BYTE              SFLocAddr[6];   /* Local  CP address (hex)           */
      BYTE              SFRemAddr[6];   /* Remote CP address (hex)           */
   } SF_CC_CSIF;

/* Subfield Type */
#define SF_CC_TYPE_CSIF                 0x04


/*****************************************************************************/
/*                            Sock Subfield                                  */
/*****************************************************************************/

/* This sf added to provide parms required by TBIRD socket under a TBIRD B   */
/* environment. The absence of this SF on a CC.REQ or CC.RSP implies a TBIRD */
/* A (no flows) configuration                                                */
typedef STRUCT16 SF_CC_Sock
   {
      WORD              SFLength;       /* sizeof(SF_CC_CSIF)                */
      BYTE              SFIdentifier;   /* SF_SFID_CALL_CONTROL              */
      BYTE              SFType;         /* SF_CC_TYPE_SOCK                   */
      WORD              SFNodeType;     /* Node Type                         */
      BYTE              SFAddr[6];      /* local PDFH address to exchange    */
   } SF_CC_SOCK;

/* Subfield Type */
#define SF_CC_TYPE_SOCK                 0x05

#define NODE_TYPE_DOS_WKS               0x01
#define NODE_TYPE_OS2_WKS               0x02
#define NODE_TYPE_OS2_SVR               0x03


/*****************************************************************************/
/*                            PCM_Port_Selection Subfield                    */
/*****************************************************************************/

typedef BYTE SF_PORTNAME[16];

// This subfield indicates the PCM/Port to send the call
typedef STRUCT16 SF_CC_PCM_Port_Selection
{
      WORD              SFLength;         // sizeof(SF_CC_PCM_PORT_SELECTION)
      BYTE              SFIdentifier;     // SF_SFID_CALL_CONTROL
      BYTE              SFType;           // SF_CC_TYPE_PCM_PORT_SELECTION
      BYTE              SFPCMName[16];    // PCM Name
      WORD              SFReserved;       // Reserved
      WORD              SFNumberOfPorts;  // Number of PortName
      SF_PORTNAME       SFPortName[1];    // PortName
} SF_CC_PCM_PORT_SELECTION;

/* Subfield Type */
#define SF_CC_TYPE_PCM_PORT_SELECTION   0x06





/*****************************************************************************/
/*                        Identifier Subfield Pointers                       */
/*****************************************************************************/

#ifndef _SF_HEADER
typedef STRUCT16 SF_Header
{
   WORD              SFLength;         /* sizeof(SF_HEADER)-1+subfield data */
   BYTE              SFIdentifier;     /* SF_SFID_*                         */
   BYTE              SFType;           /* SF_*_TYPE_**                      */
} SF_HEADER;
#define _SF_HEADER
#endif

/* Generic Subfield Pointer Union */
typedef union
{
   /* Generic Subfields */
   BYTE                        PTR16 pByte;
   SF_HEADER                   PTR16 pHeader;

   /* CCITT Subfields */
   SF_CC_BEAR_CAP              PTR16 pBearCap;
   SF_CC_BEAR_CAP_ISDN         PTR16 pBearCapISDN;
   SF_CC_CHANNEL_ID            PTR16 pChannelID;
   SF_CC_CHANNEL_ID_ISDN       PTR16 pChannelIDISDN;
   SF_CC_CALLING_NUMBER        PTR16 pCallingNumber;
   SF_CC_CALLED_NUMBER         PTR16 pCalledNumber;
   SF_CC_CALLING_SUBADDRESS    PTR16 pCallingSubaddress;
   SF_CC_CALLED_SUBADDRESS     PTR16 pCalledSubaddress;
   SF_CC_LOW_LAYER_COMP        PTR16 pLowLayerComp;
   SF_CC_HIGH_LAYER_COMP       PTR16 pHighLayerComp;
   SF_CC_USER_USER_INFO        PTR16 pUserUserInfo;
   SF_CC_USER_FACILITY         PTR16 pUserFacility;
   SF_CC_PROGRESS_IND          PTR16 pProgressInd;
   SF_CC_DISPLAY               PTR16 pDisplay;
   SF_CC_CAUSE                 PTR16 pCause;

   /* Network Specific Subfields*/
   SF_NS_CHARGING_INFO         PTR16 pChargingInfo;
   SF_NS_DATE                  PTR16 pDate;

   /* Call Control Subfields */
   SF_CS_CALL_STATUS           PTR16 pCallStatus;
   SF_CS_RAW_MSG               PTR16 pRawMsg;
   SF_CS_PERMANENT             PTR16 pPermanent;
   SF_CC_CSIF                  PTR16 pCSIF;
   SF_CC_SOCK                  PTR16 pSock;
   SF_CC_PCM_PORT_SELECTION    PTR16 pPCMPort;

} SF_SUBFIELD;

#endif
